/* @start_prolog@
 * Version: %Z% %W% %I% %E% %U%  
 * ============================================================================
 *   <copyright 
 *   notice="oco-source" 
 *   pids="5724-H72," 
 *   years="2010,2012" 
 *   crc="2914692611" > 
 *   IBM Confidential 
 *    
 *   OCO Source Materials 
 *    
 *   5724-H72, 
 *    
 *   (C) Copyright IBM Corp. 2010, 2012 
 *   Copyright 2020 MicroEJ Corp. This file has been modified by MicroEJ Corp.
 *    
 *   The source code for the program is not published 
 *   or otherwise divested of its trade secrets, 
 *   irrespective of what has been deposited with the 
 *   U.S. Copyright Office. 
 *   </copyright> 
 * ============================================================================
 * @end_prolog@
 */
package com.cwm.net;

import com.cwm.util.GPRSConnectOptions;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.net.Socket;
import javax.net.SocketFactory;
import org.eclipse.paho110.client.mqttv311.MqttException;
import org.eclipse.paho110.client.mqttv311.internal.NetworkModule;
import org.eclipse.paho110.client.mqttv311.logging.Logger;
import org.eclipse.paho110.client.mqttv311.logging.LoggerFactory;


/**
 * A network module for connecting over TCP from Java ME (CLDC profile).
 */
public class TCPMicroNetworkModule implements NetworkModule {
    private String uri;
    private String host;
    private int port;
    private Socket connection;
    private InputStream in;
    private OutputStream out;
    private GPRSConnectOptions connOptions;
    static final String className = TCPMicroNetworkModule.class.getName();
    Logger log = LoggerFactory.getLogger(LoggerFactory.MQTT_CLIENT_MSG_CAT,className);
    
    /**
     * Constructs a new MicroTCPNetworkModule using the specified host and
     * port.
     * 
     * @param host the host name to connect to
     * @param port the port to connect to
     * @param secure whether or not to use SSL.
     */
    public TCPMicroNetworkModule(String host, int port) {
            connOptions = GPRSConnectOptions.getInstance();
            
            if (connOptions.getConnParamsSocket() != null) {
                this.uri = "socket://" + host + ":" + port + ";" + connOptions.getConnParamsSocket();
            } else {
                this.uri = "socket://" + host + ":" + port
                        + ";bearer_type=" + connOptions.getBearerType()
                        + ";access_point=" + connOptions.getAPN()
                        + ";username=" + connOptions.getUser()
                        + ";password=" + connOptions.getPasswd()
                        + ";timeout=" + connOptions.getTimeout();
            }
            this.host = host;
            this.port = port;
    }
    
    /**
     * Starts the module, by creating a TCP socket to the server.
     */
    public void start() throws IOException, MqttException {
        final String methodName = "start";
        try {
            log.fine(className,methodName, "252", new Object[] {uri});
            connection = SocketFactory.getDefault().createSocket();
            connection.connect(new InetSocketAddress(host, port), connOptions.getTimeout());
            connection.setSoLinger(true, 5); 
            in = connection.getInputStream();
            out = connection.getOutputStream();
        }
        catch (IOException ex) {
            //@TRACE 250=Failed to create TCP socket
            log.fine(className,methodName,"250",null,ex);
            throw new MqttException(MqttException.REASON_CODE_SERVER_CONNECT_ERROR, ex);
        }
    }

    public InputStream getInputStream() {
        return in;
    }

    public OutputStream getOutputStream() {
        return out;
    }

    /**
     * Stops the module, by closing the TCP socket.
     */
    public void stop() throws IOException {
        in.close();
        out.close();
        connection.close();
    }
    
    public String getServerURI() {              // SJM - added for Paho project version 1.1.0
        String retVal;
        
        if (this.uri != null) {
            retVal = this.uri;
        } else {
            retVal = "";
        }
        
        return  retVal;
    }    
}
