/*
 * C
 *
 * Copyright 2020 MicroEJ Corp.
 * This software has been designed or modified by MicroEJ Corp.
 * MicroEJ Corp. grants to THALES the non-exclusive right to freely use, modify and distribute this software.
 *
 */

/**
 * @file
 * @brief Generic LLBSP implementation.
 * @author MicroEJ Developer Team
 * @version 1.0.0
 * @date 11 April 2018
 */

#include <stdint.h>
#include <stdio.h>
#include <string.h>

#include "LLBSP_impl.h"
#include "gina.h"

#ifdef __cplusplus
extern "C" {
#endif

#define BUFFER_SIZE (512)

static uint8_t buffer_out[BUFFER_SIZE];
static uint32_t buffer_index = 0;

extern void SOAR_START(void);
extern void SOAR_END(void);

#ifdef MICROEJ_MULTIAPP
extern void _java_installed_features_text_start(void);
extern void _java_installed_features_text_size(void);
#endif // MICROEJ_MULTIAPP

/**
 * Checks if the given pointer is in a read only memory or not.
 */
uint8_t LLBSP_IMPL_isInReadOnlyMemory(void* ptr) {
	uint32_t addr = (uint32_t) ptr;
	uint32_t soarStart = (uint32_t)(&SOAR_START);
	uint32_t soarEnd = (uint32_t)(&SOAR_END);
#ifdef MICROEJ_MULTIAPP
	uint32_t installedFeaturesStart = (uint32_t)(&_java_installed_features_text_start);
	uint32_t installedFeaturesEnd = installedFeaturesStart + (uint32_t)(&_java_installed_features_text_size);
#endif // MICROEJ_MULTIAPP
	return (uint8_t)(((addr >= soarStart) && (addr < soarEnd))
#ifdef MICROEJ_MULTIAPP
			|| ((addr >= installedFeaturesStart) && (addr < installedFeaturesEnd))
#endif // MICROEJ_MULTIAPPl
			);
}

/*
 * Writes the character <code>c</code>, cast to an unsigned char, to stdout stream.
 * This function is used by the default implementation of the Java <code>System.out</code>.
 */
void LLBSP_IMPL_putchar(int32_t c) {
	if (('\n' == c) || ((BUFFER_SIZE - 1) <= buffer_index)) {
		gina_uwlog_printf("%s", buffer_out);
		buffer_index = 0;
		memset((void*)&buffer_out, 0, BUFFER_SIZE);
	}
	else {
		buffer_out[buffer_index] = c;
		buffer_index++;
	}
}

#ifdef __cplusplus
}
#endif
