/*
 * C
 *
 * Copyright 2020 MicroEJ Corp.
 * This software has been designed or modified by MicroEJ Corp.
 * MicroEJ Corp. grants to THALES the non-exclusive right to freely use, modify and distribute this software.
 *
 */

/**
 * @file
 * @brief MicroEJ startup.
 * @author MicroEJ Developer Team
 * @version 1.0.0
 * @date 24 April 2020
 */

#include <stdio.h>
#include "microej_main.h"
#include "LLMJVM.h"
#include "sni.h"
#include "gina.h"

#ifdef __cplusplus
    extern "C" {
#endif

/**
 * @brief Creates and starts a MicroEJ instance. This function returns when the MicroEJ execution ends.
 */
void microej_main(void) {
	void* vm;
	int32_t err;
	int32_t exitcode;

	// create VM
	vm = SNI_createVM();

	if (vm == NULL) {
		gina_uwlog_printf("MicroEJ initialization error.");
	} else {
		gina_uwlog_printf("MicroEJ START");
		err = SNI_startVM(vm, 0, NULL);

		if (err < 0) {
			// Error occurred
			if (err == LLMJVM_E_EVAL_LIMIT) {
				gina_uwlog_printf("Evaluation limits reached.");
			} else {
				gina_uwlog_printf("MicroEJ execution error (err = %d).", (int) err);
			}
		} else {
			// VM execution ends normally
			exitcode = SNI_getExitCode(vm);
			gina_uwlog_printf("MicroEJ END (exit code = %d)", (int) exitcode);
		}

		// delete VM
		SNI_destroyVM(vm);
	}
}

#ifdef __cplusplus
    }
#endif
