/*
 * C
 *
 * Copyright 2020 MicroEJ Corp.
 * This software has been designed or modified by MicroEJ Corp.
 * MicroEJ Corp. grants to THALES the non-exclusive right to freely use, modify and distribute this software.
 *
 */

#include <stdint.h>

#include "microej_time.h"
#include "gina.h"
#include "qapi_timer.h"

#ifdef __cplusplus
extern "C" {
#endif

#define MILLISECONDS_TO_NANOSECONDS (1000000)

/**
 * @brief Gets the time since the module started.
 * 
 * @return uint64_t The time since start in milliseconds.
 */
static uint64_t get_time_milliseconds(void) {
	qapi_Status_t status = 0;
	qapi_time_get_t time_get;

	status = qapi_time_get(QAPI_TIME_MSECS, &time_get);
	if (QAPI_OK != status) {
		gina_uwlog_printf("%s: Error getting time, status=%d", __func__, status);
		return 0;
	}

	return time_get.time_msecs;
}

/**
 * @brief Initializes the hardware timer for MicroEJ.
 * 
 */
void microej_timer_initialize(void) {
	/* Nothing to do here. */
}

/**
 * @brief Gets the current time in milliseconds.
 *
 * @return int64_t The current time in milliseconds.
 */
int64_t microej_timer_get_current_time(void) {
	return (int64_t) get_time_milliseconds();
}

/**
 * @brief Gets the current time in nanoseconds.
 *
 * @return int64_t The current time in nanoseconds.
 */
int64_t microej_timer_get_time_nanos(void) {
	return (int64_t) get_time_milliseconds() * MILLISECONDS_TO_NANOSECONDS;
}

#ifdef __cplusplus
}
#endif
