/*
 * C
 *
 * Copyright 2020 MicroEJ Corp.
 * This software has been designed or modified by MicroEJ Corp.
 * MicroEJ Corp. grants to THALES the non-exclusive right to freely use, modify and distribute this software.
 */

/**
 * @file
 * @brief SSL low level implementation configuration and macros
 * @author MicroEJ Developer Team
 * @version 1.0.0
 * @date 16 September 2020
 */

#ifndef  LLNET_SSL_CONFIGURATION_H
#define  LLNET_SSL_CONFIGURATION_H

#include "qapi_ssl.h"
#include "LLNET_SSL_ERRORS.h"

// comment this define to remove secure network debug traces
// #define LLNET_SSL_DEBUG

#undef LLNET_SSL_DEBUG_TRACE
#undef LLNET_SSL_DEBUG_ASSERT

#ifdef LLNET_SSL_DEBUG
#include <stdio.h>
#include "gina.h"
#define LLNET_SSL_DEBUG_TRACE       gina_uwlog_printf
#define LLNET_SSL_DEBUG_ASSERT(...) gina_uwlog_printf(__VA_ARGS__); while(1);
#else
#define LLNET_SSL_DEBUG_TRACE(...)  ((void) 0)
#define LLNET_SSL_DEBUG_ASSERT(...) ((void) 0)
#endif // LLNET_SSL_DEBUG

#define J_CREATE_SSL_SOCKET_ERROR   J_CREATE_SSL_ERROR

#define SSL_AVAILABLE_BUFFER_SIZE      (4096)
#define LLNET_SSL_MAX_CERTIFICATE_SIZE (4096)
#define LLNET_SSL_MAX_PASSWORD_SIZE    (64)

typedef struct LLNET_SSL_context {
	qapi_Net_SSL_Obj_Hdl_t handle;
	int32_t protocol;
	uint8_t certificate_buffer[LLNET_SSL_MAX_CERTIFICATE_SIZE];
	int32_t certificate_buffer_len;
	uint8_t password[LLNET_SSL_MAX_PASSWORD_SIZE]; // Null-terminated string
	qapi_Net_SSL_Cert_Info_t cert_chain_info;
	int32_t cert_in_chain;
	uint8_t* cert_chain;
	int32_t cert_chain_offset;
} LLNET_SSL_context_t;

typedef struct LLNET_SSL_socket {
	qapi_Net_SSL_Con_Hdl_t connection;
	int32_t fd;
	uint8_t available_buffer[SSL_AVAILABLE_BUFFER_SIZE];
	int32_t available;
} LLNET_SSL_socket_t;

#endif // LLNET_SSL_CONFIGURATION_H
