/*
 * C
 *
 * Copyright 2020 MicroEJ Corp.
 * This software has been designed or modified by MicroEJ Corp.
 * MicroEJ Corp. grants to THALES the non-exclusive right to freely use, modify and distribute this software.
 *
 */

#ifndef OSAL_PORTMACRO_H
#define OSAL_PORTMACRO_H

/**
 * @file
 * @brief OS Abstraction Layer ThreadX port macro
 * @author MicroEJ Developer Team
 * @version 1.0.0
 * @date 12 May 2020
 */

#include <stdint.h>
#include "gina.h"

/**
 * @brief Uncomment this if ThreadX Modules is used.
 * 
 */
#define THREADX_MODULES

/** @brief Custom OS type definitions */
#define OSAL_CUSTOM_TYPEDEF

/** @brief task function entry point */
typedef void OSAL_task_entry_point_t(uint32_t id);

/** @brief OS task handle */
typedef struct _OSAL_task_handle_t {
	TX_THREAD* tx_thread;
 } OSAL_task_handle_t;

/** @brief OS queue handle */
typedef struct _OSAL_queue_handle_t {
	TX_QUEUE* tx_queue;
} OSAL_queue_handle_t;

/** @brief OS counter semaphore handle */
typedef struct _OSAL_counter_semaphore_handle_t {
	TX_SEMAPHORE* tx_semaphore_counter;
} OSAL_counter_semaphore_handle_t;

/** @brief OS binary semaphore handle */
typedef struct _OSAL_binary_semaphore_handle_t {
	TX_SEMAPHORE* tx_semaphore;
} OSAL_binary_semaphore_handle_t;

/** @brief OS mutex handle */
typedef struct _OSAL_mutex_handle_t {
	TX_MUTEX* tx_mutex;
 } OSAL_mutex_handle_t;

/** @brief OS task stack */
typedef struct _OSAL_task_stack_t {
	int32_t size;
	uint8_t *stack;
} OSAL_task_stack_t;

/** @brief OS queue */
typedef struct _OSAL_queue_t {
	int32_t size;
	void *addr;
} OSAL_queue_t;

/**
 * @brief Declare a task stack.
 *
 * @param[in] _name name of the variable that defines the stack.
 * @param[in] _size size of the stack in bytes. _size must be compile time constant value.
 */
#define OSAL_task_stack_declare(_name, _size) uint8_t _name ## _buffer[_size]; OSAL_task_stack_t _name = { .size = _size, .stack = _name ## _buffer }

/**
 * @brief Declare a queue.
 *
 * @param[in] _name name of the variable that defines the queue address.
 * @param[in] _size number of items that can be stored in the queue. _size must be compile time constant value.
 */
#define OSAL_queue_declare(_name, _size) void *_name ## _buffer = microej_malloc(_size); OSAL_queue_t _name = { .size = _size, .addr = _name ## _buffer }

#endif // OSAL_PORTMACRO_H
