/*
 * C
 *
 * Copyright 2020 MicroEJ Corp.
 * This software has been designed or modified by MicroEJ Corp.
 * MicroEJ Corp. grants to THALES the non-exclusive right to freely use, modify and distribute this software.
 *
 */

#include <stdint.h>
#include <stdio.h>

#include "gina.h"
#include "tx_api.h"
#include "qapi_timer.h"
#include "txm_module.h"

extern void osal_threadx_test_all(void);
extern void qapi_spi_master_test_all(void);

/**
 * @brief Callback for the hello world timer.
 * 
 * @param userData 
 */
void timer_signal_cb(uint32_t userData) {
	tx_event_flags_set((TX_EVENT_FLAGS_GROUP *) userData, 1, TX_OR);
}

/**
 * @brief Prints the message hello world.
 * 
 */
void hello_world_loop(void) {
	qapi_TIMER_define_attr_t time_define_attr;
	qapi_TIMER_handle_t dog_timer;
	TX_EVENT_FLAGS_GROUP *event = NULL;
	qapi_Status_t result = QAPI_ERROR;
	int counter = 1;

	// create a wait event
	result = txm_module_object_allocate(&event, sizeof(TX_EVENT_FLAGS_GROUP));
	if (result != QAPI_OK) {
		GINA_UWLOG_CRITICAL("txm_module_object_allocate: result=%i", result);
		goto ERROR;
	}
	result = tx_event_flags_create(event, "APPWAIT");
	if (result != QAPI_OK) {
		GINA_UWLOG_CRITICAL("tx_event_flags_create: result=%i", result);
		goto ERROR;
	}


// FIXME: CC0231G-50 Link issue with display
uint8_t __icetea___6bss_6microui_6display_6externalFontsHeap$$Base;
uint8_t __icetea___6bss_6microui_6display_6externalFontsHeap$$Limit;

extern void osal_threadx_test_all(void);

void cleanup(void* data) {
	gina_cleanup();
	gina_uwlog_printf("Cleanup: %s", (gchar*) data);
}

/**
 * @brief Entry point for the embedded application.
 * 
 * @return int32_t 
 */
int32_t _dam_main(void) {
	uint8_t status = 0;

	gina_init();

	// /* set the output to URC, "^SUSRW" will be displayed on the instance
	//  on which userware was started with at^srusw */
	// gina_uwlog_set_dst(GINA_UWLOG_DST_URC);

	GINA_UWLOG_INFO("DAM main");

	hello_world_loop();

	/* Call your test here */
	gina_uwlog_printf("Starting tests");
	
#ifdef OSAL_TEST
	osal_threadx_test_all();
#endif // OSAL_TEST
	qapi_spi_master_test_all();

	return 0;
}
