/*
 * C
 *
 * Copyright 2020 MicroEJ Corp.
 * This software has been designed or modified by MicroEJ Corp.
 * MicroEJ Corp. grants to THALES the non-exclusive right to freely use, modify and distribute this software.
 *
 */

#include <stdint.h>

#include "gina.h"

//#define OSAL_TEST
#ifdef OSAL_TEST

#include "osal.h"

void test_semaphore(void) {
	OSAL_binary_semaphore_handle_t handle = { 0 };
	OSAL_status_t status = OSAL_ERROR;
	uint32_t count = 1;

	status = OSAL_binary_semaphore_create((uint8_t*) "TEST SEMAPHORE", count, &handle);
	if (OSAL_OK != status) {
		gina_uwlog_printf("%s: OSAL_binary_semaphore_create error, status = %d", __func__, status);
		return;
	}
	gina_uwlog_printf("%s: OSAL_binary_semaphore_create success", __func__);

	status = OSAL_binary_semaphore_give(&handle);
	if (OSAL_OK != status) {
		gina_uwlog_printf("%s: OSAL_binary_semaphore_give error, status = %d", __func__, status);
		return;
	}
	gina_uwlog_printf("%s: OSAL_binary_semaphore_give success", __func__);

	status = OSAL_binary_semaphore_delete(&handle);
	if(OSAL_OK != status) {
		gina_uwlog_printf("%s: OSAL_binary_semaphore_delete error, status = %d", __func__, status);
		return;
	}
	gina_uwlog_printf("%s: OSAL_binary_semaphore_delete success", __func__);
}

uint8_t task_finished = 0;

void task_entry_point(uint32_t parameters) {
	gina_uwlog_printf("Task entry point");
	task_finished = 1;
}

OSAL_task_stack_declare(task_stack, 4096);

void test_thread(void) {
	OSAL_task_handle_t handle = { 0 };
	OSAL_status_t status = 0;

	status = OSAL_task_create(task_entry_point, (uint8_t*) "MY TASK", task_stack, 185, NULL, &handle);
	if (OSAL_OK != status) {
		gina_uwlog_printf("%s: OSAL_task_create error, status = %d", __func__, status);
		return;
	}
	gina_uwlog_printf("%s: OSAL_task_create success", __func__);

	while (!task_finished) {
		tx_thread_sleep(100);
	}

	status = OSAL_task_delete(&handle);
	if (OSAL_OK != status) {
		gina_uwlog_printf("%s: OSAL_task_delete error, status = %d", __func__, status);
		return;
	}
	gina_uwlog_printf("%s: OSAL_task_delete success", __func__);

	OSAL_task_handle_t out = { 0 };
	status = OSAL_task_get_current(&out);
	if (OSAL_OK != status) {
		gina_uwlog_printf("%s: OSAL_task_get_current error, status = %d", __func__, status);
		return;
	}
	gina_uwlog_printf("%s: OSAL_task_get_current success", __func__);
}

void test_mutex(void) {
	OSAL_mutex_handle_t handle = { 0 };
	OSAL_status_t status = 0;

	status = OSAL_mutex_create((uint8_t*) "TEST MUTEX", &handle);
	if (OSAL_OK != status) {
		gina_uwlog_printf("%s: OSAL_mutex_create error, status = %d", __func__, status);
		return;
	}
	gina_uwlog_printf("%s: OSAL_mutex_create success", __func__);

	status = OSAL_mutex_take(&handle, OSAL_INFINITE_TIME);
	if (OSAL_OK != status) {
		gina_uwlog_printf("%s: OSAL_mutex_take error, status = %d", __func__, status);
		return;
	}
	gina_uwlog_printf("%s: OSAL_mutex_take success", __func__);

	status = OSAL_mutex_give(&handle);
	if (OSAL_OK != status) {
		gina_uwlog_printf("%s: OSAL_mutex_give error, status = %d", __func__, status);
		return;
	}
	gina_uwlog_printf("%s: OSAL_mutex_give success", __func__);

	status = OSAL_mutex_delete(&handle);
	if (OSAL_OK != status) {
		gina_uwlog_printf("%s: OSAL_mutex_delete error, status = %d", __func__, status);
		return;
	}
	gina_uwlog_printf("%s: OSAL_mutex_delete success", __func__);
}

void osal_threadx_test_all(void) {
	gina_uwlog_printf("osal_threadx_test_all");
	test_thread();
}
